%{?python_enable_dependency_generator}
%global debug_package %{nil}
%global with_selinux 1
%global selinuxtype targeted
%global policy_version 38.1.0

Summary:        Open source TPM software for Bootstrapping and Maintaining Trust
Name:           keylime
Version:        7.11.0
Release:        1%{?dist}
URL:            https://github.com/keylime/keylime
License:        Apache-2.0 AND MIT
Source0:        https://github.com/keylime/keylime/archive/refs/tags/v%{version}.tar.gz
Source1:        keylime.sysusers
Source2:        https://github.com/RedHat-SP-Security/%{name}-selinux/archive/v%{policy_version}/keylime-selinux-%{policy_version}.tar.gz

Patch0001:      0001-Include-Authority-Key-Identifier-in-KL-generated-cer.patch
Patch0002:      0002-ca_impl_openssl-Mark-extensions-as-critical-followin.patch
Patch0003:      0003-revocation_notifier-Use-web_util-to-generate-TLS-con.patch

BuildRequires:  git-core swig openssl-devel systemd-rpm-macros
BuildRequires:  python3-devel python3-dbus python3-jinja2 python3-setuptools

Requires: python3-keylime = %{version}-%{release}
Requires: keylime-base = %{version}-%{release}
Requires: keylime-verifier = %{version}-%{release}
Requires: keylime-registrar = %{version}-%{release}
Requires: keylime-tenant = %{version}-%{release}
Requires: keylime-tools = %{version}-%{release}

Suggests: keylime-agent

%description
Keylime is a TPM based highly scalable remote boot attestation
and runtime integrity measurement solution.


%package base
Summary: The base package contains the default configuration
License: MIT

Requires(pre): tpm2-tss python3-jinja2 shadow-utils
Requires: openssl procps-ng

%ifarch %efi
Requires: efivar-libs
%endif

%if 0%{?with_selinux}
# This ensures that the *-selinux package and all it’s dependencies are not pulled
# into containers and other systems that do not use SELinux
Recommends: (keylime-selinux if selinux-policy-%{selinuxtype})
%endif

%description base
The base package contains the Keylime default configuration


%package -n python3-keylime
Summary: The Python Keylime module
License: MIT

Requires: keylime-base = %{version}-%{release}
%{?python_provide:%python_provide python3-keylime}

Requires: python3-tornado python3-sqlalchemy python3-alembic python3-cryptography
Requires: python3-pyyaml python3-packaging python3-requests python3-gpg python3-lark-parser
Requires: python3-pyasn1 python3-pyasn1-modules python3-jsonschema python3-typing-extensions
Requires: tpm2-tools

%description -n python3-keylime
The python3-keylime module implements the functionality used
by Keylime components.


%package verifier
Summary: The Python Keylime Verifier component
License: MIT

Requires: keylime-base = %{version}-%{release}
Requires: python3-keylime = %{version}-%{release}

%description verifier
The Keylime Verifier continuously verifies the integrity state
of the machine that the agent is running on.


%package registrar
Summary: The Keylime Registrar component
License: MIT

Requires: keylime-base = %{version}-%{release}
Requires: python3-keylime = %{version}-%{release}

%description registrar
The Keylime Registrar is a database of all agents registered
with Keylime and hosts the public keys of the TPM vendors.


%if 0%{?with_selinux}
# SELinux subpackage
%package selinux
Summary:             keylime SELinux policy
BuildArch:           noarch
Requires:            selinux-policy-%{selinuxtype}
Requires(post):      selinux-policy-%{selinuxtype}
BuildRequires:       selinux-policy-devel
%{?selinux_requires}

%description selinux
Custom SELinux policy module
%endif


%package tenant
Summary: The Python Keylime Tenant
License: MIT

Requires: keylime-base = %{version}-%{release}
Requires: python3-keylime = %{version}-%{release}

%description tenant
The Keylime Tenant can be used to provision a Keylime Agent.


%package tools
Summary: Keylime tools
License: MIT

Requires: keylime-base = %{version}-%{release}
Requires: python3-keylime = %{version}-%{release}

%description tools
The keylime tools package includes miscelaneous tools.


%prep
%autosetup -S git -n keylime-%{version} -a2


%build
%if 0%{?with_selinux}
# SELinux policy (originally from selinux-policy-contrib)
# this policy module will override the production module
make -f %{_datadir}/selinux/devel/Makefile keylime.pp
bzip2 -9 keylime.pp
%endif
%py3_build


%install
%py3_install
mkdir -p %{buildroot}/%{_sharedstatedir}/keylime
mkdir -p --mode=0700 %{buildroot}/%{_rundir}/keylime

mkdir -p --mode=0700 %{buildroot}/%{_sysconfdir}/keylime/
for comp in "verifier" "tenant" "registrar" "ca" "logging"; do
    mkdir -p --mode=0700  %{buildroot}/%{_sysconfdir}/keylime/${comp}.conf.d
    install -Dpm 400 config/${comp}.conf %{buildroot}/%{_sysconfdir}/keylime
done

# Ship some scripts.
mkdir -p %{buildroot}/%{_datadir}/keylime/scripts
for s in create_runtime_policy.sh \
         create_mb_refstate \
         ek-openssl-verify; do
    install -Dpm 755 scripts/${s} \
        %{buildroot}/%{_datadir}/keylime/scripts/${s}
done

# Ship configuration templates.
cp -r ./templates %{buildroot}%{_datadir}/keylime/templates/

mkdir -p --mode=0755 %{buildroot}/%{_bindir}
install -Dpm 755 ./keylime/cmd/convert_config.py %{buildroot}/%{_bindir}/keylime_upgrade_config

%if 0%{?with_selinux}
install -D -m 0644 keylime.pp.bz2 %{buildroot}%{_datadir}/selinux/packages/%{selinuxtype}/keylime.pp.bz2
install -D -p -m 0644 keylime-selinux-%{policy_version}/keylime.if %{buildroot}%{_datadir}/selinux/devel/include/distributed/keylime.if
%endif

install -Dpm 644 ./services/keylime_verifier.service \
    %{buildroot}%{_unitdir}/keylime_verifier.service

install -Dpm 644 ./services/keylime_registrar.service \
    %{buildroot}%{_unitdir}/keylime_registrar.service

cp -r ./tpm_cert_store %{buildroot}%{_sharedstatedir}/keylime/

install -p -d %{buildroot}/%{_tmpfilesdir}
cat > %{buildroot}/%{_tmpfilesdir}/keylime.conf << EOF
d %{_rundir}/keylime 0700 keylime keylime -
EOF

install -p -D -m 0644 %{SOURCE1} %{buildroot}%{_sysusersdir}/keylime.conf


%pre base
%sysusers_create_compat %{SOURCE1}
usermod -aG tss keylime || :
exit 0

%post base
/usr/bin/keylime_upgrade_config --component ca --component logging >/dev/null
exit 0

%posttrans base
if [ -d %{_sysconfdir}/keylime ]; then
    chmod 500 %{_sysconfdir}/keylime
    chown -R keylime:keylime %{_sysconfdir}/keylime

    for comp in "verifier" "tenant" "registrar" "ca" "logging"; do
        [ -d %{_sysconfdir}/keylime/${comp}.conf.d ] && \
            chmod 500 %{_sysconfdir}/keylime/${comp}.conf.d
    done
fi

[ -d %{_sharedstatedir}/keylime ] && \
    chown -R keylime %{_sharedstatedir}/keylime/

[ -d %{_sharedstatedir}/keylime/tpm_cert_store ] && \
    chmod 400 %{_sharedstatedir}/keylime/tpm_cert_store/*.pem && \
    chmod 500 %{_sharedstatedir}/keylime/tpm_cert_store/

[ -d %{_localstatedir}/log/keylime ] && \
    chown -R keylime %{_localstatedir}/log/keylime/
exit 0


%post verifier
/usr/bin/keylime_upgrade_config --component verifier >/dev/null
%systemd_post keylime_verifier.service

%preun verifier
%systemd_preun keylime_verifier.service

%postun verifier
%systemd_postun_with_restart keylime_verifier.service


%post registrar
/usr/bin/keylime_upgrade_config --component registrar >/dev/null
%systemd_post keylime_registrar.service

%preun registrar
%systemd_preun keylime_registrar.service

%postun registrar
%systemd_postun_with_restart keylime_registrar.service


%post tenant
/usr/bin/keylime_upgrade_config --component tenant >/dev/null
exit 0

%preun tenant
%systemd_preun keylime_registrar.service


%if 0%{?with_selinux}
# SELinux contexts are saved so that only affected files can be
# relabeled after the policy module installation
%pre selinux
%selinux_relabel_pre -s %{selinuxtype}

%post selinux
%selinux_modules_install -s %{selinuxtype} %{_datadir}/selinux/packages/%{selinuxtype}/keylime.pp.bz2
%selinux_relabel_post -s %{selinuxtype}

if [ "$1" -le "1" ]; then # First install
    # The services need to be restarted for the custom label to be
    # applied in case they where already present in the system,
    # restart fails silently in case they where not.
    for svc in registrar verifier; do
        [ -f "%{_unitdir}/keylime_${svc}".service ] && \
            %systemd_postun_with_restart "keylime_${svc}".service
    done
fi
exit 0

%postun selinux
if [ $1 -eq 0 ]; then
    %selinux_modules_uninstall -s %{selinuxtype} keylime
    %selinux_relabel_post -s %{selinuxtype}
fi
%endif


%files verifier
%license LICENSE
%attr(500,keylime,keylime) %dir %{_sysconfdir}/keylime/verifier.conf.d
%config(noreplace) %verify(not md5 size mode mtime) %attr(400,keylime,keylime) %{_sysconfdir}/keylime/verifier.conf
%{_bindir}/keylime_verifier
%{_bindir}/keylime_ca
%{_unitdir}/keylime_verifier.service

%files registrar
%license LICENSE
%attr(500,keylime,keylime) %dir %{_sysconfdir}/keylime/registrar.conf.d
%config(noreplace) %verify(not md5 size mode mtime) %attr(400,keylime,keylime) %{_sysconfdir}/keylime/registrar.conf
%{_bindir}/keylime_registrar
%{_unitdir}/keylime_registrar.service

%if 0%{?with_selinux}
%files selinux
%{_datadir}/selinux/packages/%{selinuxtype}/keylime.pp.*
%{_datadir}/selinux/devel/include/distributed/keylime.if
%ghost %verify(not md5 size mode mtime) %{_sharedstatedir}/selinux/%{selinuxtype}/active/modules/200/keylime
%endif

%files tenant
%license LICENSE
%attr(500,keylime,keylime) %dir %{_sysconfdir}/keylime/tenant.conf.d
%config(noreplace) %verify(not md5 size mode mtime) %attr(400,keylime,keylime) %{_sysconfdir}/keylime/tenant.conf
%{_bindir}/keylime_tenant

%files -n python3-keylime
%license LICENSE
%{python3_sitelib}/keylime-*.egg-info/
%{python3_sitelib}/keylime
%{_datadir}/keylime/scripts/create_mb_refstate
%{_bindir}/keylime_attest
%{_bindir}/keylime_convert_runtime_policy
%{_bindir}/keylime_create_policy
%{_bindir}/keylime_sign_runtime_policy

%files tools
%license LICENSE
%{_bindir}/keylime_userdata_encrypt

%files base
%license LICENSE
%doc README.md
%attr(500,keylime,keylime) %dir %{_sysconfdir}/keylime/{ca,logging}.conf.d
%config(noreplace) %verify(not md5 size mode mtime) %attr(400,keylime,keylime) %{_sysconfdir}/keylime/ca.conf
%config(noreplace) %verify(not md5 size mode mtime) %attr(400,keylime,keylime) %{_sysconfdir}/keylime/logging.conf
%attr(700,keylime,keylime) %dir %{_rundir}/keylime
%attr(700,keylime,keylime) %dir %{_sharedstatedir}/keylime
%attr(500,keylime,keylime) %dir %{_sharedstatedir}/keylime/tpm_cert_store
%attr(400,keylime,keylime) %{_sharedstatedir}/keylime/tpm_cert_store/*.pem
%{_tmpfilesdir}/keylime.conf
%{_sysusersdir}/keylime.conf
%{_datadir}/keylime/scripts/create_runtime_policy.sh
%{_datadir}/keylime/scripts/ek-openssl-verify
%{_datadir}/keylime/templates
%{_bindir}/keylime_upgrade_config

%files
%license LICENSE


%changelog
*  Tue Jan 14 2025 Sinong Chen <costinchen@tencent.com> - 7.11.0-1
- initial build
